@@ -41,6 +41,6 @@ extern char *mlt_property_get_string( mlt_property self );
 extern void *mlt_property_get_data( mlt_property self, int *length );
 extern void mlt_property_close( mlt_property self );
 
-extern void mlt_property_pass( mlt_property this, mlt_property that );
+extern void mlt_property_pass( mlt_property self, mlt_property that );
 
 #endif
