@@ -45,7 +45,7 @@ static void rgba_bgra( uint8_t *src, uint8_t* dst, int width, int height )
 	}	
 }
 
-int process_frei0r_item( mlt_service_type type, double position, mlt_properties prop, mlt_frame this, uint8_t **image, int *width, int *height )
+int process_frei0r_item( mlt_service service, double position, double time, mlt_properties prop, mlt_frame this, uint8_t **image, int *width, int *height )
 {
 	int i=0;
 	f0r_instance_t ( *f0r_construct ) ( unsigned int , unsigned int ) =  mlt_properties_get_data(  prop , "f0r_construct" ,NULL);
@@ -57,8 +57,9 @@ int process_frei0r_item( mlt_service_type type, double position, mlt_properties
 	void (*f0r_set_param_value)(f0r_instance_t instance, f0r_param_t param, int param_index)=mlt_properties_get_data(  prop , "f0r_set_param_value" ,NULL);
 	void (*f0r_update2) (f0r_instance_t instance, double time,
 			 const uint32_t* inframe1,const uint32_t* inframe2,const uint32_t* inframe3,
-	 uint32_t* outframe)=mlt_properties_get_data(  prop , "f0r_update2" ,NULL);
-
+	uint32_t* outframe)=mlt_properties_get_data(  prop , "f0r_update2" ,NULL);
+	mlt_service_type type = mlt_service_identify( service );
+	int not_thread_safe = mlt_properties_get_int( prop, "_not_thread_safe" );
 
 	//use as name the width and height
 	f0r_instance_t inst;
@@ -66,6 +67,8 @@ int process_frei0r_item( mlt_service_type type, double position, mlt_properties
 	char ctorname[1024]="";
 	sprintf(ctorname,"ctor-%dx%d",*width,*height);
 
+	mlt_service_lock( service );
+
 	void* neu=mlt_properties_get_data( prop , ctorname ,NULL );
 	if (!f0r_construct){
 		//printf("no ctor\n");
@@ -77,6 +80,10 @@ int process_frei0r_item( mlt_service_type type, double position, mlt_properties
 	}else{
 		inst=mlt_properties_get_data( prop ,  ctorname , NULL );
 	}
+
+	if ( !not_thread_safe )
+		mlt_service_unlock( service );
+	
 	if (f0r_get_plugin_info){
 		f0r_get_plugin_info(&info);
 		for (i=0;i<info.num_params;i++){
@@ -136,17 +143,19 @@ int process_frei0r_item( mlt_service_type type, double position, mlt_properties
 		}
 	}
 	if (type==producer_type) {
-		f0r_update (inst, position, NULL, dest );
+		f0r_update (inst, time, NULL, dest );
 	} else if (type==filter_type) {
-		f0r_update ( inst, position, source[0], dest );
+		f0r_update ( inst, time, source[0], dest );
 	} else if (type==transition_type && f0r_update2 ){
-		f0r_update2 ( inst, position, source[0], source[1], NULL, dest );
+		f0r_update2 ( inst, time, source[0], source[1], NULL, dest );
 	}
+	if ( not_thread_safe )
+		mlt_service_unlock( service );
 	if (info.color_model == F0R_COLOR_MODEL_BGRA8888) {
 		rgba_bgra((uint8_t*) dest, (uint8_t*) result, *width, *height);
 	}
 	*image = (uint8_t*) result;
-	mlt_properties_set_data(MLT_FRAME_PROPERTIES(this), "image", result, video_area * sizeof(uint32_t), mlt_pool_release, NULL);
+	mlt_frame_set_image(this, (uint8_t*) result, video_area * sizeof(uint32_t), mlt_pool_release);
 	if (extra)
 		mlt_pool_release(extra);
 
