@@ -164,6 +164,10 @@ saved_rack_parse_plugin (jack_rack_t * jack_rack, saved_rack_t * saved_rack, sav
   xmlChar *content;
   unsigned long num;
   unsigned long control = 0;
+#ifdef WIN32
+  xmlFreeFunc xmlFree = NULL;
+  xmlMemGet( &xmlFree, NULL, NULL, NULL);
+#endif
 
   for (node = plugin->children; node; node = node->next)
     {
@@ -257,6 +261,10 @@ saved_rack_parse_jackrack (jack_rack_t * jack_rack, saved_rack_t * saved_rack, c
   xmlNodePtr node;
   xmlChar *content;
   saved_plugin_t * saved_plugin;
+#ifdef WIN32
+  xmlFreeFunc xmlFree = NULL;
+  xmlMemGet( &xmlFree, NULL, NULL, NULL);
+#endif
 
   for (node = jackrack->children; node; node = node->next)
     {
