@@ -120,7 +120,7 @@ static int filter_get_image( mlt_frame frame, uint8_t **image, mlt_image_format
 	mlt_properties frame_properties = MLT_FRAME_PROPERTIES(frame);
 
 	// Get the frame position
-	mlt_position position = mlt_frame_get_position( frame );
+	mlt_position position = mlt_filter_get_position( filter, frame );
 
 	// Get the new image
 	int error = mlt_frame_get_image( frame, image, format, width, height, 1 );
@@ -128,6 +128,8 @@ static int filter_get_image( mlt_frame frame, uint8_t **image, mlt_image_format
 	if( error != 0 )
 		mlt_properties_debug( frame_properties, "error after mlt_frame_get_image() in autotrack_rectangle", stderr );
 
+	mlt_service_lock( MLT_FILTER_SERVICE( filter ) );
+
 	// Get the geometry object
 	mlt_geometry geometry = mlt_properties_get_data(filter_properties, "filter_geometry", NULL);
 
@@ -138,6 +140,14 @@ static int filter_get_image( mlt_frame frame, uint8_t **image, mlt_image_format
 	// Get the motion vectors
 	struct motion_vector_s *vectors = mlt_properties_get_data( frame_properties, "motion_est.vectors", NULL );
 
+	// Cleanse the geometry item
+	boundry.w = boundry.x < 0 ? boundry.w + boundry.x : boundry.w;
+	boundry.h = boundry.y < 0 ? boundry.h + boundry.y : boundry.h;
+	boundry.x = boundry.x < 0 ? 0 : boundry.x;
+	boundry.y = boundry.y < 0 ? 0 : boundry.y;
+	boundry.w = boundry.w < 0 ? 0 : boundry.w;
+	boundry.h = boundry.h < 0 ? 0 : boundry.h;
+
 	// How did the rectangle move?
 	if( vectors != NULL &&
 	    boundry.key != 1 ) // Paused?
@@ -163,7 +173,9 @@ static int filter_get_image( mlt_frame frame, uint8_t **image, mlt_image_format
 		mlt_geometry_insert(geometry, &boundry);
 	}
 
-       	if( mlt_properties_get_int( filter_properties, "debug" ) == 1 )
+	mlt_service_unlock( MLT_FILTER_SERVICE( filter ) );
+
+	if( mlt_properties_get_int( filter_properties, "debug" ) == 1 )
 	{
 		init_arrows( format, *width, *height );
 		draw_rectangle_outline(*image, boundry.x, boundry.y, boundry.w, boundry.h, 100);
@@ -203,8 +215,10 @@ static int attach_boundry_to_frame( mlt_frame frame, uint8_t **image, mlt_image_
 	mlt_properties frame_properties = MLT_FRAME_PROPERTIES(frame);
 
 	// Get the frame position
-	mlt_position position = mlt_frame_get_position( frame );
+	mlt_position position = mlt_filter_get_position( filter, frame );
 	
+	mlt_service_lock( MLT_FILTER_SERVICE( filter ) );
+
 	// Get the geometry object
 	mlt_geometry geometry = mlt_properties_get_data(filter_properties, "filter_geometry", NULL);
 	if (geometry == NULL) {
@@ -224,10 +238,20 @@ static int attach_boundry_to_frame( mlt_frame frame, uint8_t **image, mlt_image_
 		geometry = mlt_properties_get_data(filter_properties, "filter_geometry", NULL);
 	}
 
+	mlt_service_unlock( MLT_FILTER_SERVICE( filter ) );
+
 	// Get the current geometry item
 	mlt_geometry_item geometry_item = mlt_pool_alloc( sizeof( struct mlt_geometry_item_s ) );
 	mlt_geometry_fetch(geometry, geometry_item, position);
 
+	// Cleanse the geometry item
+	geometry_item->w = geometry_item->x < 0 ? geometry_item->w + geometry_item->x : geometry_item->w;
+	geometry_item->h = geometry_item->y < 0 ? geometry_item->h + geometry_item->y : geometry_item->h;
+	geometry_item->x = geometry_item->x < 0 ? 0 : geometry_item->x;
+	geometry_item->y = geometry_item->y < 0 ? 0 : geometry_item->y;
+	geometry_item->w = geometry_item->w < 0 ? 0 : geometry_item->w;
+	geometry_item->h = geometry_item->h < 0 ? 0 : geometry_item->h;
+
 	mlt_properties_set_data( frame_properties, "bounds", geometry_item, sizeof( struct mlt_geometry_item_s ), mlt_pool_release, NULL );
 
 	// Get the new image
