@@ -72,10 +72,11 @@ static int filter_get_image( mlt_frame this, uint8_t **image, mlt_image_format *
 	mlt_filter filter = mlt_frame_pop_service( this );
 
 	// Get the image
+	*format = mlt_image_yuv422;
 	int error = mlt_frame_get_image( this, image, format, width, height, 1 );
 
 	// Only process if we have no error and a valid colour space
-	if ( error == 0 && *format == mlt_image_yuv422 )
+	if ( error == 0 )
 	{
 		// Get the charcoal scatter value
 		int x_scatter = mlt_properties_get_double( MLT_FILTER_PROPERTIES( filter ), "x_scatter" );
@@ -137,7 +138,7 @@ static int filter_get_image( mlt_frame this, uint8_t **image, mlt_image_format *
 		*image = temp;
 
 		// Store new and destroy old
-		mlt_properties_set_data( MLT_FRAME_PROPERTIES( this ), "image", *image, *width * *height * 2, mlt_pool_release, NULL );
+		mlt_frame_set_image( this, *image, *width * *height * 2, mlt_pool_release );
 	}
 
 	return error;
