@@ -63,13 +63,8 @@ static mlt_geometry transition_parse_keys( mlt_transition this, const char *name
 	// Try to fetch it first
 	mlt_geometry geometry = mlt_properties_get_data( properties, store, NULL );
 
-	// Get the in and out position
-	int always_active = mlt_properties_get_int( properties, "always_active" );
-	mlt_position in = mlt_transition_get_in( this );
-	mlt_position out = !always_active ? mlt_transition_get_out( this ) : -1;
-
 	// Determine length and obtain cycle
-	int length = out - in + 1;
+	mlt_position length = mlt_transition_get_length( this );
 	double cycle = mlt_properties_get_double( properties, "cycle" );
 
 	// Allow a geometry repeat cycle
@@ -347,11 +342,14 @@ static inline void get_affine( affine_t *affine, mlt_transition this, float posi
 		float shear_x = composite_calculate_key( this, "shear_x", "shear_x_info", 360, position );
 		float shear_y = composite_calculate_key( this, "shear_y", "shear_y_info", 360, position );
 		float shear_z = composite_calculate_key( this, "shear_z", "shear_z_info", 360, position );
-
+		float o_x = composite_calculate_key( this, "ox", "ox_info", 0, position );
+		float o_y = composite_calculate_key( this, "oy", "oy_info", 0, position );
+		
 		affine_rotate_x( affine->matrix, rotate_x );
 		affine_rotate_y( affine->matrix, rotate_y );
 		affine_rotate_z( affine->matrix, rotate_z );
 		affine_shear( affine->matrix, shear_x, shear_y, shear_z );
+		affine_offset( affine->matrix, o_x, o_y );
 	}
 }
 
@@ -381,19 +379,18 @@ static int transition_get_image( mlt_frame a_frame, uint8_t **image, mlt_image_f
 	int b_width;
 	int b_height;
 
-	// Get the unique name to retrieve the frame position
-	char *name = mlt_properties_get( properties, "_unique_id" );
+	// Assign the current position
+	mlt_position position =  mlt_transition_get_position( this, a_frame );
 
-	// Assign the current position to the name
-	mlt_position position =  mlt_properties_get_position( a_props, name );
-	
-	mlt_properties props = mlt_properties_get_data( b_props, "_producer", NULL );
-	int always_active = mlt_properties_get_int( properties, "always_active" );
-
-	mlt_position in = !always_active ? mlt_properties_get_position( properties, "in" ) : mlt_properties_get_int( props, "in" );
-	mlt_position out = !always_active ? mlt_properties_get_position( properties, "out" ) : mlt_properties_get_int( props, "out" );
 	int mirror = mlt_properties_get_position( properties, "mirror" );
-	int length = out - in + 1;
+	int length = mlt_transition_get_length( this );
+	if ( mlt_properties_get_int( properties, "always_active" ) )
+	{
+		mlt_properties props = mlt_properties_get_data( b_props, "_producer", NULL );
+		mlt_position in = mlt_properties_get_int( props, "in" );
+		mlt_position out = mlt_properties_get_int( props, "out" );
+		length = out - in + 1;
+	}
 
 	// Obtain the normalised width and height from the a_frame
 	int normalised_width = mlt_properties_get_int( a_props, "normalised_width" );
@@ -412,7 +409,9 @@ static int transition_get_image( mlt_frame a_frame, uint8_t **image, mlt_image_f
 	mlt_frame_get_image( a_frame, image, format, width, height, 1 );
 
 	// Calculate the region now
+	mlt_service_lock( MLT_TRANSITION_SERVICE( this ) );
 	composite_calculate( this, &result, normalised_width, normalised_height, ( float )position );
+	mlt_service_unlock( MLT_TRANSITION_SERVICE( this ) );
 
 	// Fetch the b frame image
 	result.w = ( result.w * *width / normalised_width );
@@ -550,13 +549,6 @@ static int transition_get_image( mlt_frame a_frame, uint8_t **image, mlt_image_f
 
 static mlt_frame transition_process( mlt_transition transition, mlt_frame a_frame, mlt_frame b_frame )
 {
-	// Get a unique name to store the frame position
-	char *name = mlt_properties_get( MLT_TRANSITION_PROPERTIES( transition ), "_unique_id" );
-
-	// Assign the current position to the name
-	mlt_properties a_props = MLT_FRAME_PROPERTIES( a_frame );
-	mlt_properties_set_position( a_props, name, mlt_frame_get_position( a_frame ) - mlt_transition_get_in( transition ) );
-
 	// Push the transition on to the frame
 	mlt_frame_push_service( a_frame, transition );
 
