@@ -22,12 +22,13 @@
 #include <framework/mlt_frame.h>
 #include <framework/mlt_factory.h>
 #include <framework/mlt_producer.h>
+#include <framework/mlt_log.h>
 #include <stdlib.h>
 #include <string.h>
 #include <pthread.h>
 #include <SDL.h>
 #include <SDL_syswm.h>
-#include <assert.h>
+#include <sys/time.h>
 
 extern pthread_mutex_t mlt_sdl_mutex;
 
@@ -92,6 +93,7 @@ mlt_consumer consumer_sdl_preview_init( mlt_profile profile, mlt_service_type ty
 		mlt_properties_set( properties, "real_time", "0" );
 		mlt_properties_set( properties, "rescale", "nearest" );
 		mlt_properties_set( properties, "deinterlace_method", "onefield" );
+		mlt_properties_set_int( properties, "prefill", 1 );
 		parent->close = consumer_close;
 		parent->start = consumer_start;
 		parent->stop = consumer_stop;
@@ -196,10 +198,10 @@ static int consumer_start( mlt_consumer parent )
 		mlt_properties_set_int( play, "progressive", progressive );
 		mlt_properties_set_int( still, "progressive", progressive );
 
-		mlt_properties_pass_list( play, properties, "resize,rescale,width,height,aspect_ratio,display_ratio,volume" );
-		mlt_properties_pass_list( still, properties, "resize,rescale,width,height,aspect_ratio,display_ratio" );
-		mlt_properties_pass_list( play, properties, "deinterlace_method" );
-		mlt_properties_pass_list( still, properties, "deinterlace_method" );
+		mlt_properties_pass_list( play, properties,
+			"deinterlace_method,resize,rescale,width,height,aspect_ratio,display_ratio,volume,real_time,buffer,prefill" );
+		mlt_properties_pass_list( still, properties,
+			"deinterlace_method,resize,rescale,width,height,aspect_ratio,display_ratio" );
 		mlt_properties_pass_list( play, properties, "preview_off,preview_format,window_background" );
 		mlt_properties_pass_list( still, properties, "preview_off,preview_format,window_background" );
 
@@ -249,7 +251,9 @@ static int consumer_stop( mlt_consumer parent )
 		pthread_mutex_lock( &this->refresh_mutex );
 		pthread_cond_broadcast( &this->refresh_cond );
 		pthread_mutex_unlock( &this->refresh_mutex );
+#ifndef WIN32
 		if ( this->thread )
+#endif
 			pthread_join( this->thread, NULL );
 		this->joined = 1;
 
@@ -344,6 +348,7 @@ static void *consumer_thread( void *arg )
 				mlt_position duration = mlt_producer_get_playtime( producer );
 				int pause = 0;
 
+#ifndef SKIP_WAIT_EOS
 				if ( this->active == this->play )
 				{
 					// Do not interrupt the play consumer near the end
@@ -358,10 +363,11 @@ static void *consumer_thread( void *arg )
 					}
 					else
 					{
-						// Send frame with speed 0 once to stop it
-						if ( frame && !eos && speed == 0.0 )
+						// Send frame with speed 0 to stop it
+						if ( frame && !mlt_consumer_is_stopped( this->play ) )
 						{
 							mlt_consumer_put_frame( this->play, frame );
+							frame = NULL;
 							eos = 1;
 						}
 
@@ -369,11 +375,21 @@ static void *consumer_thread( void *arg )
 						if ( mlt_consumer_is_stopped( this->play ) )
 						{
 							// Stream has ended
+							mlt_log_verbose( MLT_CONSUMER_SERVICE( consumer ), "END OF STREAM\n" );
 							pause = 1;
-							eos = 0; // reset eof indicator
+							eos = 0; // reset eos indicator
+						}
+						else
+						{
+							// Prevent a tight busy loop
+							struct timespec tm = { 0, 100000L }; // 100 usec
+							nanosleep( &tm, NULL );
 						}
 					}
 				}
+#else
+				pause = this->active == this->play;
+#endif
 				if ( pause )
 				{
 					// Start the still consumer
