@@ -751,6 +751,10 @@ static int consumer_start( mlt_consumer this )
 			int length = 0;
 			xmlDocDumpMemoryEnc( doc, &buffer, &length, "utf-8" );
 			mlt_properties_set( properties, resource, _s(buffer) );
+#ifdef WIN32
+			xmlFreeFunc xmlFree = NULL;
+			xmlMemGet( &xmlFree, NULL, NULL, NULL);
+#endif
 			xmlFree( buffer );
 		}
 		else
