@@ -1095,6 +1095,10 @@ static void on_end_property( deserialise_context context, const xmlChar *name )
 			// Serialise the tree to get value
 			xmlDocDumpMemory( context->value_doc, &value, &size );
 			mlt_properties_set( properties, context->property, _s(value) );
+#ifdef WIN32
+			xmlFreeFunc xmlFree = NULL;
+			xmlMemGet( &xmlFree, NULL, NULL, NULL);
+#endif
 			xmlFree( value );
 			xmlFreeDoc( context->value_doc );
 			context->value_doc = NULL;
@@ -1571,7 +1575,7 @@ mlt_producer producer_xml_init( mlt_profile profile, mlt_service_type servtype,
 		for ( i = mlt_properties_count( properties ) - 1; i >= 1; i -- )
 		{
 			char *name = mlt_properties_get_name( properties, i );
-			if ( mlt_properties_get_data( properties, name, NULL ) == service )
+			if ( mlt_properties_get_data_at( properties, i, NULL ) == service )
 			{
 				mlt_properties_set_data( properties, name, service, 0, NULL, NULL );
 				break;
